import os,csv,json,re
import qgis
import qgis.core
from qgis.utils import iface
from qgis.gui import QgsMessageBar
from qgis.core.additions.edit import edit
import processing

def uniqueValues(table):
    if  table.endswith('.csv'):
        uri = 'file:///'+ table +'?type=csv&skipEmptyFields=Yes&maxFields=10000&detectTypes=yes&geomType=none&subsetIndex=yes'
        layer = qgis.core.QgsVectorLayer(uri, os.path.splitext(os.path.basename(table))[0], 'delimitedtext')
        qgis.core.QgsProject.instance().addMapLayer(layer)
        prov= layer.dataProvider() 
        fields=[field.name() for field in prov.fields()]
        iface.messageBar().pushMessage("Success",  "File Variable List successfully loaded", level=3, duration=3)
        return fields
    elif table:
        layer=qgis.core.QgsVectorLayer(table, os.path.splitext(os.path.basename(table))[0])
        prov= layer.dataProvider() 
        fields=[field.name() for field in prov.fields()]
        iface.messageBar().pushMessage("Success",  "File Variable List successfully loaded", level=3, duration=3)
        return fields
    else:
        iface.messageBar().pushMessage("Critical Error",  "Field Variable List is empty", level=2, duration=3)

def textFile(table, renameFile):
    if renameFile:
        with open(table, 'r') as infile, open(renameFile, 'w') as outfile:
            stripped = (line.strip() for line in infile)
            lines = (re.split('[, *]|[*|*]|[* *]|[*,*]|[*;*]|[*:*]|[*\t*]', line) for line in stripped if line)
            lines = [[item for item in row if item != ''] for row in lines]
            writer = csv.writer(outfile)
            writer.writerows(lines)
            infile.close()
            outfile.close()
        return renameFile
    else:
        iface.messageBar().pushMessage("Critical Error", "Text File Variable List not loaded or new filename provided", level=2, duration=3)
             
def getPath(file):
    if os.path.isfile(file):
        extension=os.path.splitext(file)[1]
        i = 0
        while True:
            i += 1
            path = "{}{}{}".format(file.split(extension)[0],i,extension)
            if os.path.isfile(path):
                continue
            else:
                return path
                break
    else:
        return file

def mergeSpatial(csvFile, csvField, shpFile, shpField, mergeFile):
    uri = 'file:///'+ csvFile +'?type=csv&skipEmptyFields=Yes&maxFields=10000&detectTypes=yes&geomType=none&subsetIndex=yes'
    shp = qgis.core.QgsVectorLayer(shpFile, os.path.splitext(os.path.basename(shpFile))[0], 'ogr')
    csv = qgis.core.QgsVectorLayer(uri, os.path.splitext(os.path.basename(csvFile))[0], 'delimitedtext')
    parameters={ 'DISCARD_NONMATCHING' : True, 'FIELD' : shpField, 'FIELDS_TO_COPY' : [], 'FIELD_2' : csvField, 'INPUT' : shp, 'INPUT_2' : csv, 'METHOD' : 1, 'OUTPUT' : 'TEMPORARY_OUTPUT', 'PREFIX' : '', '--overwrite': True }
    j=processing.tools.general.run("qgis:joinattributestable",parameters)
    feats = [feat.geometry() for feat in j['OUTPUT'].getFeatures()]
    if feats:
        if csvField==shpField:
            k=processing.tools.general.run('qgis:deletecolumn', {'INPUT': j["OUTPUT"], 'COLUMN': csvField+"_2", 'OUTPUT': 'TEMPORARY_OUTPUT'})
        else:
            k=processing.tools.general.run('qgis:deletecolumn', {'INPUT': j["OUTPUT"], 'COLUMN': csvField, 'OUTPUT': 'TEMPORARY_OUTPUT'})
        qgis.core.QgsVectorFileWriter.writeAsVectorFormat(k["OUTPUT"], mergeFile, "utf-8", qgis.core.QgsCoordinateReferenceSystem("EPSG:4326"), "geojson")
        iface.addVectorLayer(mergeFile,os.path.splitext(os.path.basename(mergeFile))[0],"ogr")
        iface.messageBar().pushMessage("Success",  "{} created".format(os.path.basename(mergeFile)), level=3, duration=3)
        return mergeFile
    else:
        iface.messageBar().pushMessage("Critical Error", "Features contained Null Geometry and were not merged", level=2, duration=3)   

def geojsonFile(fileName,shp,renameFile=None):
    if renameFile:
        geojson=toGeojson(fileName, shp, renameFile)
        return geojson
    elif fileName and renameFile is None:
        iface.addVectorLayer(fileName,os.path.splitext(os.path.basename(fileName))[0],"ogr")
        return fileName
    else:
        iface.messageBar().pushMessage("Critical Error", "Spatial File Variable List not loaded correctly", level=2, duration=3)
        
def toGeojson(fileName, shp, renameFile):
    if renameFile:
        parameter={ 'INPUT' : shp, 'OPERATION' : '+proj=noop', 'OUTPUT' : 'TEMPORARY_OUTPUT', 'TARGET_CRS' : qgis.core.QgsCoordinateReferenceSystem("EPSG:4326") }
        p=processing.run('native:reprojectlayer', parameter)
        if fileName.endswith('.kml'):
            p['OUTPUT'].dataProvider().deleteAttributes([1,2,3,4,5,6,7,8,9,10,11])
            p['OUTPUT'].updateFields()
        elif fileName.endswith('.gpkg'):
            p['OUTPUT'].dataProvider().deleteAttributes([0])
            p['OUTPUT'].updateFields()
        qgis.core.QgsVectorFileWriter.writeAsVectorFormat(p['OUTPUT'], renameFile, "utf-8", qgis.core.QgsCoordinateReferenceSystem("EPSG:4326"), "geojson")
        iface.addVectorLayer(renameFile,os.path.splitext(os.path.basename(renameFile))[0],"ogr") 
        iface.messageBar().pushMessage("Success", "{} set and converted to {}".format(os.path.basename(fileName),os.path.basename(renameFile)), level=3, duration=3)
        return renameFile 
    else:
        iface.messageBar().pushMessage("Critical Error", "Spatial File not converted to GeoJSON", level=2, duration=3)  

def jsonSave(inFile,outFile):
    with open(inFile, 'r') as r, open(outFile, 'w') as w:
        d = json.load(r)
        g = json.dumps(d)
        w.write('var ' + os.path.basename(outFile)[:-3] + " = {};".format(g))
    r.close()
    w.close()
    return outFile