# -*- coding: utf-8 -*-
"""
/***************************************************************************
 dataLeafletJSDockWidget
                                 A QGIS plugin
 This program converts csv files to JavaScript files
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2020-11-22
        git sha              : $Format:%H$
        copyright            : (C) 2020 by Nathaniel Geyer
        email                : nrg139@psu.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os,re
from qgis.PyQt import QtGui, QtWidgets, uic
from qgis.PyQt.QtCore import pyqtSignal
from PyQt5.QtWidgets import QApplication, QMainWindow, QStyle, QFileDialog, QDialog, QMessageBox, QAbstractButton, QSizePolicy
from PyQt5.QtGui import QDoubleValidator, QIntValidator, QIcon
from PyQt5.QtCore import QVariant, QCoreApplication
from PyQt5.Qt import Qt

import qgis
import qgis.core
from qgis.utils import iface
from qgis.gui import QgsMessageBar
from .core_functions import *

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'data_leafletjs_dockwidget_base.ui'))

class dataLeafletJSDockWidget(QtWidgets.QDockWidget, FORM_CLASS):

    closingPlugin = pyqtSignal()

    def __init__(self, parent=None):
        """Constructor."""
        super(dataLeafletJSDockWidget, self).__init__(parent)
        # Set up the user interface from Designer.
        # After setupUI you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://doc.qt.io/qt-5/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)
        
        #set icons for folder and exit buttons icons made possible by flaticon.com
        self.areaTB.setIcon(QIcon(":/plugins/data_leafletjs/polygon.png"))  
        self.clearTB.setIcon(QIcon(":/plugins/data_leafletjs/clear.png"))
        self.geojsonTB.setIcon(QIcon(":/plugins/data_leafletjs/globe.png"))
        self.jsInTB.setIcon(QIcon(":/plugins/data_leafletjs/leaflet.jfif"))
        self.textTB.setIcon(QIcon(":/plugins/data_leafletjs/table.png")) 
        
        #connect signals
        self.areaTB.clicked.connect(self.selectSpatialFile)
        self.clearTB.clicked.connect(self.clearFileNames)
        self.geojsonTB.clicked.connect(self.selectGeojsonFile)
        self.jsInTB.clicked.connect(self.selectJSFile)
        self.textTB.clicked.connect(self.selectTextFile)
        self.mergePB.clicked.connect(self.mergeGeojsonFiles)
        self.convertPB.clicked.connect(self.createJSFile) 

    def closeEvent(self, event):
        self.closingPlugin.emit()
        event.accept()
    
    def selectSpatialFile(self):
        fileName, _ = QFileDialog.getOpenFileName(self,"Select file as","","Spatial File (*.geojson *.kml *.shp *.gpkg)")
        shp = qgis.core.QgsVectorLayer(fileName, os.path.splitext(os.path.basename(fileName))[0], 'ogr')
        if fileName.endswith('.geojson') and shp.crs() == qgis.core.QgsCoordinateReferenceSystem("EPSG:4326") and fileName:
            geojson=geojsonFile(fileName,shp)
            if geojson:
                slist=uniqueValues(geojson)
                if slist:
                    self.areaLE.setText(geojson)
                    self.areaCB.addItems(slist)
        elif fileName.endswith(('.kml','.shp', '.gpkg','.geojson')) or shp.crs() != qgis.core.QgsCoordinateReferenceSystem("EPSG:4326") and fileName:
            renameFile, _ = QFileDialog.getSaveFileName(self,"Select file", getPath(os.path.splitext(fileName)[0]+".geojson"),"GEOJSON File (*.geojson)")
            geojson=geojsonFile(fileName,shp,renameFile)
            if geojson:
                slist=uniqueValues(geojson)
                if slist:
                    self.areaLE.setText(geojson)
                    self.areaCB.addItems(slist)
        else:
            iface.messageBar().pushMessage("Critical Error", "Spatial File Variable List not loaded correctly", level=2, duration=3)         
            
    def selectTextFile(self):
        fileName, _ = QFileDialog.getOpenFileName(self,"Select file", "","Delimited Text File (*.csv *.txt)")
        if fileName.endswith('.csv') and bool(re.search(r'\d+', open(fileName).read().rstrip("\n"))) and fileName:
            clist=uniqueValues(fileName)
            if clist:
                self.textLE.setText(fileName)
                self.textCB.addItems(clist)            
        elif fileName.endswith('.txt') and bool(re.search(r'\d+', open(fileName).read().rstrip("\n"))) and bool(re.match(r'[A-Za-z]', open(fileName).read().rstrip("\n"))):
            renameFile, _ = QFileDialog.getSaveFileName(self,"Select file", getPath(os.path.splitext(fileName)[0]+".csv"),"Delimited Text File (*.csv)")   
            renameFile=textFile(fileName,renameFile)
            if renameFile:
                clist=uniqueValues(renameFile)
                if clist:
                    self.textLE.setText(renameFile)
                    self.textCB.addItems(clist)
        else:
            iface.messageBar().pushMessage("Critical Error", "Text File Variable List not loaded, or filename provided", level=2, duration=3)
    
    def selectGeojsonFile(self):
        fileName, _ = QFileDialog.getOpenFileName(self,"Select file as","","Spatial File (*.geojson *.kml *.shp *.gpkg)")
        shp = qgis.core.QgsVectorLayer(fileName, os.path.splitext(os.path.basename(fileName))[0], 'ogr')
        jsFile=self.jsInLE.text()
        if fileName.endswith('.geojson') and shp.crs() == qgis.core.QgsCoordinateReferenceSystem("EPSG:4326") and fileName:
            geojson=geojsonFile(fileName,shp)
            if geojson and jsFile:
                self.geojsonToLE.setText(fileName)
                self.geojsonCoLE.setText(geojson)
                outFile=jsonSave(geojson,jsFile)
                self.jsOutLE.setText(outFile)
                iface.messageBar().pushMessage("Success", "{} set; {} created".format(os.path.basename(geojson),os.path.basename(outFile)), level=3, duration=3)
            elif geojson:
                self.geojsonToLE.setText(fileName)
                self.geojsonCoLE.setText(geojson)
                iface.messageBar().pushMessage("Success", "{} set".format(os.path.basename(fileName)), level=3, duration=3)
        elif fileName.endswith(('.kml','.shp', '.gpkg', '.geojson')) and shp.crs() != qgis.core.QgsCoordinateReferenceSystem("EPSG:4326") and fileName:
            renameFile, _ = QFileDialog.getSaveFileName(self,"Select file", getPath(os.path.splitext(fileName)[0]+".geojson"),"GEOJSON File (*.geojson)")
            geojson=geojsonFile(fileName,shp,renameFile)
            if geojson and jsFile:
                self.geojsonToLE.setText(fileName)
                self.geojsonCoLE.setText(geojson)
                outFile=jsonSave(geojson,jsFile)
                self.jsOutLE.setText(outFile)
                iface.messageBar().pushMessage("Success", "{} set; {} created".format(os.path.basename(geojson),os.path.basename(jsFile)), level=3, duration=3)
            elif geojson:
                self.geojsonToLE.setText(fileName)
                self.geojsonCoLE.setText(geojson)
                iface.messageBar().pushMessage("Success", "{} set".format(os.path.basename(fileName)), level=3, duration=3)
        else:
            iface.messageBar().pushMessage("Critical Error", "Spatial File not provided", level=2, duration=3)
    
    def selectJSFile(self):
        fileName, _ = QFileDialog.getSaveFileName(self,"Select file", "","JavaScript File (*.js)")
        inFile=self.geojsonCoLE.text()
        if fileName and inFile:
            self.jsInLE.setText(fileName)
            outFile=jsonSave(inFile,fileName)
            self.jsOutLE.setText(outFile)
            iface.messageBar().pushMessage("Success", "{} set; {} created".format(os.path.basename(fileName),os.path.basename(outFile)), level=3, duration=3)
        elif fileName:
            self.jsInLE.setText(fileName)
            iface.messageBar().pushMessage("Success", "{} set".format(os.path.basename(fileName)), level=3, duration=3)
        else:
            iface.messageBar().pushMessage("Critical Error",  "JavaScript File not set or provided", level=2, duration=3)       
           
    def mergeGeojsonFiles(self):
        csvFile = self.textLE.text()
        csvField = self.textCB.currentText()
        shpFile = self.areaLE.text()
        shpField = self.areaCB.currentText()
        if all([csvFile, csvField, shpFile, shpField]):
            mergeFile, _ = QFileDialog.getSaveFileName(self,"Select file", getPath(os.path.splitext(csvFile)[0]+".geojson"),"GEOJSON File (*.geojson)")
            if mergeFile:
                self.geojsonToLE.setText(mergeFile)
                merged=mergeSpatial(csvFile, csvField, shpFile, shpField, mergeFile)
                self.geojsonCoLE.setText(merged)
            else:
                iface.messageBar().pushMessage("Critical Error",  "Merge files not set", level=2, duration=3) 
        else:
            iface.messageBar().pushMessage("Critical Error",  "Merge and text files not set", level=2, duration=3)    

    def createJSFile(self):
        inFile=self.geojsonCoLE.text()
        jsFile=self.jsInLE.text()
        if inFile and not jsFile:
            jsFile, _ = QFileDialog.getSaveFileName(self,"Select file", getPath(os.path.splitext(inFile)[0]+".js"),"JavaScript File (*.js)")
            self.jsInLE.setText(jsFile)
        if jsFile:
            outFile=jsonSave(inFile,jsFile)
            self.jsOutLE.setText(outFile)
            iface.messageBar().pushMessage("Success",  "{} created".format(os.path.basename(outFile)), level=3, duration=3)
        else:
            iface.messageBar().pushMessage("Critical Error",  "JavaScript files not set", level=2, duration=3)         

    def clearFileNames(self):
        clear = QMessageBox.question(self, 'Message',"Do you want to clear all the files and layers?", QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if clear == QMessageBox.Yes:
            self.areaLE.clear()
            self.textLE.clear()
            self.geojsonToLE.clear()
            self.geojsonCoLE.clear()
            self.jsInLE.clear()
            self.jsOutLE.clear()
            self.areaCB.clear()
            self.textCB.clear()
            qgis.core.QgsProject.instance().clear()
            iface.messageBar().pushMessage("Success", "Files and Layers were cleared", level=3, duration=3)
        else:
            iface.messageBar().pushMessage("Info", "Files and Layers were not cleared", level=0, duration=3)           